print("import: 'stevedore'")
import stevedore

print("import: 'stevedore.named'")
import stevedore.named

print("import: 'stevedore.driver'")
import stevedore.driver

print("import: 'stevedore.hook'")
import stevedore.hook

print("import: 'stevedore.extension'")
import stevedore.extension

print("import: 'stevedore.enabled'")
import stevedore.enabled

print("import: 'stevedore.dispatch'")
import stevedore.dispatch

print("import: 'stevedore.tests'")
import stevedore.tests

print("import: 'stevedore._cache'")
import stevedore._cache

